<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:common="org.enhydra.dods.Common"
                xmlns:dodst="org.enhydra.dods.xslt.DODSTag"
                xmlns:miu="org.enhydra.dods.xslt.XSLTUtil"
                version="1.0">
    <xsl:output method="text" encoding="iso-8859-1"/>
<xsl:template name="manyToManyReferrersDO">
<xsl:if test="@n2n='true'">
    /**
     * From the many-to-many relationship expressed by <xsl:value-of select="../@name"/>DO,
     * get array of <xsl:value-of select="dodst:getDoName(@do_name)"/>DO objects that indirectly refer
     * to this DO.
     *
     * @return Array of <xsl:value-of select="dodst:getDoName(@do_name)"/>DO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public <xsl:value-of select="@do_name"/>DO[] <xsl:value-of select="dodst:getArrayDON2N(dodst:getDoName(@do_name),../@name)"/> ()
    throws DataObjectException {
        <xsl:value-of select="@do_name"/>DO[] ret = null;
        try {
            <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] arr = get<xsl:value-of select="../@name"/>DOArray();
            ret = new <xsl:value-of select="@do_name"/>DO[ arr.length ];
            for ( int i = 0; i &lt; arr.length; i++ ) {
                ret[ i ] = arr[ i ].get<xsl:value-of select="common:capitalizeName(@name)"/>();
            }
        } catch ( Exception e ) {
            throw new DataObjectException(
            "INTERNAL ERROR: ", e );
        } finally {
            if ( null == ret )
            ret = new <xsl:value-of select="@do_name"/>DO[ 0 ];
        }
        return ret;
    }

    /**
     * To the many-to-many relationship expressed by <xsl:value-of select="../@name"/>DO,
     * add a <xsl:value-of select="dodst:getDoName(@do_name)"/>DO object that indirectly refers
     * to this DO.
     *
     * @param d The <xsl:value-of select="dodst:getDoName(@do_name)"/>DO to add to the <xsl:value-of select="../@name"/>DO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void map<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( <xsl:value-of select="@do_name"/>DO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
        // WebDocWf fix, was ambiguous, the following line was changed:
        map<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( d, (DBTransaction)null );
        // before: map<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( d, null );
        // end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by <xsl:value-of select="../@name"/>DO,
     * add a <xsl:value-of select="dodst:getDoName(@do_name)"/>DO object that indirectly refers to this DO.
     *
     * @param b The <xsl:value-of select="dodst:getDoName(@do_name)"/>DO to add to the <xsl:value-of select="../@name"/>DO mapping for this DO.
     * @param tran The transaction to be used. If null, a new transaction is created.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void map<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( <xsl:value-of select="@do_name"/>DO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO m = null;
        try {
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
            if (originDatabase != null)
                m = <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO.createVirgin(originDatabase);
            else</xsl:if>
                m = <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO.createVirgin();
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO.createVirgin failed&quot;, e );
        }
        m.set<xsl:value-of select="common:capitalizeName(@name)"/>( d );
        m.set<xsl:value-of select="common:capitalizeName(@another)"/>( this );
        m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by <xsl:value-of select="../@name"/>DO,
     * remove (delete) the <xsl:value-of select="dodst:getDoName(@do_name)"/>DO object that indirectly refers
     * to this DO.
     *
     * @param d The <xsl:value-of select="dodst:getDoName(@do_name)"/>DO to remove from the <xsl:value-of select="../@name"/>DO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmap<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( <xsl:value-of select="@do_name"/>DO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
        // WebDocWf fix, was ambiguous, the following line was changed:
        unmap<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( d, (DBTransaction)null );
        // before:  unmap<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( d, null );
        //end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by <xsl:value-of select="../@name"/>DO,
     * remove (delete) the <xsl:value-of select="dodst:getDoName(@do_name)"/>DO object that indirectly refers
     * to this DO.
     *
     * @param b The <xsl:value-of select="dodst:getDoName(@do_name)"/>DO to remove from the <xsl:value-of select="../@name"/>DO mapping
     * for this DO.
     * @param tran The transaction to be used. If null, a new transaction is created.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmap<xsl:value-of select="dodst:getDoName(@do_name)"/>_via_<xsl:value-of select="../@name"/>DO( <xsl:value-of select="@do_name"/>DO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        if (originDatabase != null)
            q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
        else</xsl:if>
            q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
        q.setQuery<xsl:value-of select="common:capitalizeName(@another)"/>( this, QueryBuilder.EQUAL );
        q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( d, QueryBuilder.EQUAL );
        q.requireUniqueInstance();
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO m = null;
        try {
            m = q.getNextDO();
        } catch ( NonUniqueQueryException e ) {
            throw new DataObjectException( &quot;Multiple mappings for &quot; +
            this + &quot; and &quot; + d + &quot; in <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/> table.&quot; );
        }
        m.delete( tran );
    }
</xsl:if>
</xsl:template>
</xsl:stylesheet>
