<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:stu="org.ejen.ext.StringUtil"
                version="1.0">
    <xsl:output method="text" encoding="iso-8859-1"/>
    <xsl:template name="fileFooter8">
<xsl:value-of select="N21_REFERRERS"/>
<xsl:value-of select="N2N_REFERRERS"/>
    /**
     * A stub method for implementing pre-commit assertions 
     * for this <xsl:value-of select="CLASS_NAME"/>DO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for writing to the database.
     */
    protected void okToCommit() 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for this <xsl:value-of select="CLASS_NAME"/>DO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for deletion from the database.
     */
    protected void okToDelete() 
    throws RefAssertionException { }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * @deprecated Use save() instead.
     */
    public void commit() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        commit(null);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     * 
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * @deprecated Use save() instead.
     */
    public void commit(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException(&quot;<xsl:value-of select="CLASS_NAME"/>DO's cache is read-only. Therefore, DML opertions are not allowed.&quot;);
        // WebDocWf extension for generic store
        // The following line has been inserted:
        try { 
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'"> else {
                if ( !dbtlocal.getDatabaseName().equals(getOriginDatabase()) ) {
                    throw new DatabaseManagerException("Logical database of transaction is different from DO's.");
                }
            }
</xsl:if>
            // The following line has been changed:
            modifyDO( dbtlocal, false );
            // before: modifyDO( dbt, false );
<xsl:if test="(TEMPLATE_SET='webdocwf' or TEMPLATE_SET='multidb_webdocwf') and DO_IS_OID_BASED='true'">
            // The following line has been inserted:
            super.save(dbtlocal);
</xsl:if>
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) {
            modifyDO( dbt, false ); 
        } 
        // end of WebDocWf extension for generic store
    }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    public void save() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(null,true);
    }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @param references True if references should be saved with this DO.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    public void save(boolean references) 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(null,references);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     * 
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    public void save(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(dbt, true);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     * 
     * @param dbt The transaction object used for this operation.
     * @param references True if references of this DO should be saved.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    public void save(DBTransaction dbt, boolean references)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException(&quot;<xsl:value-of select="CLASS_NAME"/>DO's cache is read-only. Therefore, DML opertions are not allowed.&quot;);
        // before: modifyDO( dbt, false );
        // The following line has been inserted:
        try {  // Added by Nenad Vico 25.11.2002
            // WebDocWf extension for generic store
            // The following line has been inserted:
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'"> else {
                if ( !dbtlocal.getDatabaseName().equals(getOriginDatabase()) ) {
                    throw new DatabaseManagerException("Logical database of transaction is different from DO's.");
                }
            }
</xsl:if>
            // The following line has been changed:
            modifyDO( dbtlocal, false, references );
<xsl:if test="(TEMPLATE_SET='webdocwf' or TEMPLATE_SET='multidb_webdocwf') and DO_IS_OID_BASED='true'">
            super.save(dbtlocal);
</xsl:if>
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) {  // Added by Nenad Vico 25.11.2002
            modifyDO( dbt, false );  // Added by Nenad Vico 25.11.2002
        }  // Added by Nenad Vico 25.11.2002
        // end of WebDocWf extension for generic store
    }

    /**
     * Deletes the DO from its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     */
    public void delete() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        delete((DBTransaction)null);
    }

    /**
     * Deletes the DO from its table.
     * The transaction is likely provided by the delete() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     */
    public void delete(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {

        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException(&quot;<xsl:value-of select="CLASS_NAME"/>DO's cache is read-only. Therefore, DML opertions are not allowed.&quot;);
        // WebDocWf extension for generic store
        // The following lines have been inserted:
        try {       // Added by Nenad Vico 25.11.2002
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'"> else {
                if ( !dbtlocal.getDatabaseName().equals(getOriginDatabase()) ) {
                    throw new DatabaseManagerException("Logical database of transaction is different from DO's.");
                }
            }
</xsl:if>
<xsl:if test="(TEMPLATE_SET='webdocwf' or TEMPLATE_SET='multidb_webdocwf') and DO_IS_OID_BASED='true'">
            super.delete(dbtlocal);
</xsl:if>
            // The following line has been changed:
            modifyDO( dbtlocal, true );
            // before:  modifyDO( dbt, true );
            // end of WebDocWf extension for generic store
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) { // Added by Nenad Vico 25.11.2002
            modifyDO( dbt, true ); // Added by Nenad Vico 25.11.2002
        } // Added by Nenad Vico 25.11.2002
    }
    </xsl:template>
</xsl:stylesheet>
